const http = require('http')
const url = require('url')
const fs = require('fs').promises
const { createReadStream } = require('fs')
const path = require('path')
const mime = require('mime')
const ejs = require('ejs'); // 模板引擎 
const zlib = require('zlib') //压缩
const chalk = require("chalk");
class Server {
    constructor(options) {
        this.port = options.port
        this.cwd = options.cwd

    }
    async requestHander(req, res) {
        let { pathname, query } = url.parse(req.url)
        pathname = decodeURIComponent(pathname) // 处理中文
        let filePath = path.join(this.cwd, pathname)
        try {
            let statObj = await fs.stat(filePath)
            if (statObj.isDirectory()) {
                try {
                    let indexPath = path.join(filePath, 'index.html')
                    fs.accessSync(indexPath)
                    this.sendFile(req, res, indexPath)
                } catch (error) {
                    let dirs = await fs.readdir(filePath)
                    let templateFile = await fs.readFile(path.join(__dirname, 'index.html'), 'utf-8')
                    let str = ejs.render(templateFile, {
                        arr: dirs,
                        // 如果路径是/ 那就不需要增加路径
                        currentPath: pathname === '/' ? '' : pathname
                    })
                    res.setHeader('Content-Type', 'text/html;charset=utf-8')
                    res.end(str)
                }
            }
            if (statObj.isFile()) {
                this.sendFile(req, res, filePath, statObj)
            }
        } catch (error) {
            this.errorHander(req, res, error)
        }
    }
    sendFile(req, res, filePath, statObj) {
        // 先做强制缓存
        // 做缓存，如果文件没变 直接去浏览器缓存找就ok了
        res.setHeader('Cache-Control', 'max-age=10')
        res.setHeader("Expires", new Date(Date.now() + 10 * 1000).toGMTString())
        if (this.catch(req, res, statObj, filePath)) {
            res.statusCode = 304
            return res.end()
        }
        // 返回文件之前 需要做压缩操作 ,如果能压缩 就返回一个转换流
        res.setHeader('Content-Type', mime.getType(filePath) + ";charset=utf-8")
        let gzip;
        if (gzip = this.gzip(req, res, statObj)) {
            return createReadStream(filePath).pipe(gzip).pipe(res)
        }
        createReadStream(filePath).pipe(res)
    }
    //压缩
    gzip(req, res, statObj) {
        let encoding = req.headers['accept-encoding']
        console.log(encoding)
        if (encoding) {
            if (/gzip/.test(encoding)) {
                res.setHeader('Content-Encoding', 'gzip')
                return zlib.createGzip()
            }
            if (/deflate/.test(encoding)) {
                res.setHeader('Content-Encoding', 'deflate')
                return zlib.createDeflate()
            }
        }
        return false
    }
    //对比缓存
    catch(req, res, statObj, filePath) {
        const ifModifiSince = req.headers['if-modified-since']
        const ifNoneMatch = req.headers['if-none-match']

        //获取Etag
        const currentLastModified = statObj.ctime.toGMTString()
        const currentEtag = "abc" + statObj.size

        // let fileContent = await fs.readFile(filePath, 'utf8')
        // const currentEtag = crypto.createHash('md5').update(fileContent).digest('base64')
        res.setHeader('Etag', currentEtag)
        res.setHeader('Last-Modified', currentLastModified)
        if (ifModifiSince !== currentLastModified) {
            return false
        }
        if (ifNoneMatch !== currentEtag) {
            return false
        }
        return true

    }
    errorHander(req, res, error) {
        res.statusCode = 404
        res.end(error + "")
    }
    start() {
        http.createServer(this.requestHander.bind(this)).listen(this.port, () => {
            console.log(`${chalk.yellow("Starting up http-server, serving ./")}
  Available on:
  http://127.0.0.1:${chalk.green(this.port)}
Hit CTRL-C to stop the server
    `);
        })
    }
}
// new Server({
//     port:3000,
//     cwd:process.cwd(),
//     // ...opts
// }).start()
module.exports = Server